;
; this edf_checker program is executed by the web server to check the
; *.edf files sent to be sent to the icg-ii eurochamp server
;
;	1. check the input file for errors
; 	2. read the inputfile into structure
;	3. create postscript plot
;	4. create header html file
;
; if the program exits on error
;	1. it return a non-zero exit value
; 	2. it leaves a *.check.log
;	3. creates from the log file a htmlfile to be displayed
;
; when run successfully it will produce the following files
;	1. *.edf.check.log		plain text checking info
;	2. *.edf.check.ps		post script file containing a plot
;	3. *.edf.check.pdf		pdf version fo ps file
;	4. *.edf.check.html		gives info in html format
;	5. *.edf.check.zip		contains files 1,2,3,4
;
;
FORWARD_FUNCTION _edf_utils_delete_message

PRO _edf_checker_html_error, file, result
	idx = WHERE(STRPOS(result, 'ERROR:=') EQ 0, cdx)
	idx2 = WHERE(STRPOS(result, '!!') EQ 0, cdx2)
	html_file_ascii, file+'.check.html', result $
		, /FIXED, WIDTH=800 $
		, B_INDEX=(cdx GT 0) ? idx : notdef $
		, RED_INDEX=(cdx GT 0) ? idx : notdef $
		, BLUE_INDEX=(cdx2 GT 0) ? idx2 : notdef $
		, TITLE='edf_check results for '+file $
		, /NOBREAK $
		, INFOTEXT= ['<h2>Result of the edf check for  <span style="color:blue">'+file+'</span></h2>' $
				, '<p><ul><li>The test returned <b>one or more errors</b>, please refer to text below. </li> '$
				, '<li>The <b><span style="color:red"> red lines</span></b>' $
				, ' mark where the program found the first error(s) and stopped ... </li>' $
				, '<li>There could be more problems in this data set, please <b>re-run</b> until no error occurs.</li>' $
				, '<li>Please  <B>'+create_href('http://saphir.fz-juelich.de/eurochamp/', 'klick here')$
				, '</B> to return to Eurochamp@FZJ page.</li></ul></p>']
END

FUNCTION _edf_checker_html_success, file, ERROR=err
    zipfile = file+'.check.zip' ;;;  random_file_name(NCHAR=20) + '.zip'
    htmlfile = file+'.check.html' ;;;  random_file_name(NCHAR=20) + '.zip'
    path = FILE_DIRNAME(file, /MARK)
	FILE_DELETE, zipfile, htmlfile, /ALLOW_NONEXISTENT, /QUIET
	files = FILE_BASENAME(FILE_SEARCH(file+'.*', COUNT = c))
	IF c EQ 0 THEN RETURN, 'no files'
	now = string2js(js2string(/NOW))
	hinfo=''
	h1=html_header(TITLE='Successful test of edf file! Results for '+file, NO_HR=1)
	h2f= '<li>'+create_href(files)+'</li>'
	h2f = ['<ul>',h2f,'</ul>']
	h2z = ['<ul>','<li>'+create_href(FILE_BASENAME(zipfile))+'</li>','</ul>']
	h2 = ['<table cellspacing="2" cellpadding="1" border="0" width ="700"><tr><td><hr>' $
		, '<p>The file <strong>'+FILE_BASENAME(file)+'</strong> was successfully tested by the edf_checker. ' $
		, 'Please select from the list of input/output files:</p>', h2f $
		, '<p>You can also download all files at</p>',h2z $
		, '<p>If you have problems reading the output file please go back to the ' $
		+ create_href('http://saphir.fz-juelich.de/eurochamp/','Eurochamp page at SAPHIR') $
	 	+ ' for further info.</p>' $
	 	, _edf_utils_delete_message() $
	 	, '<hr></td></tr></table>']
	h3=html_trailer(_EXTRA=e, NO_HR=1)
	html=[h1, h2, h3]
	_edf_utils_put_file, htmlfile, /TRANS, html
	files = [files, FILE_BASENAME(htmlfile)]
	IF !version.os_family EQ 'unix' THEN $
        FOR i = 0, N_ELEMENTS(files)-1 DO SPAWN, 'zip -9 -j '+zipfile+' '+path+files[i]
RETURN, 'html & zip file created ....'
END


FUNCTION _edf_checker_plot_struct, file, icg, ERROR=err, _EXTRA=e
	info = edf_checker_date(VERSION=vs, TIME=tm, COPYRIGHT=cr)
	tags = (get_tag_and_short_names(icg)).tag_name
	IF N_ELEMENTS(tags) LT 2 THEN RETURN, 'no data to plot'
	title = 'DATA of file '+file +' / plotted by edf_checker version '+vs
	IF tags[0] EQ 'TIME' THEN BEGIN
		xr = [MIN(icg.time.param), MAX(icg.time.param)] $
			+ [-0.05,0.05]*(MAX(icg.time.param)- MIN(icg.time.param))
		CASE 1 OF
			(xr[1]-xr[0]) GT 1e7 : tf ='MM.YY'
			(xr[1]-xr[0]) GT 1e5 : tf ='DD.MM'
			(xr[1]-xr[0]) LT 1E3 : tf ='HH:MM:SS'
			ELSE: tf = 'HH:MM'
		ENDCASE
		title += '!C first data record at '+js2string(icg.time.param[0])
	ENDIF ELSE xr = [0.,0.]

	mplotw, /INIT, PSFLAG=1, PSFILE=file+'.check.ps', _EXTRA=e $
		, NROW=3, NCOL=3, CHARSIZE= .7, CHARTHICK=1.4, PPOS=ppos
	FOR i=1, N_ELEMENTS(tags)-1 DO $
		mplotw, /MPLOT, /DONT, _EXTRA=e $
			, icg, NROW=3, NCOL=3, POS=ppos[*,(i-1) MOD 9] $
			, XNAME=tags[0], YNAME=tags[i], /TAG $
			, COLOR=(i MOD 5) +1, PSYM=-206, SYMSIZE=.2 $
			, CHARSIZE= .7, CHARTHICK=1.4 $
			, YNOZERO= 1, XRANGE=xr, XST=1 $
			, YERR=1, S_HAT=0, /VALID $
			, TIMEFORMAT=tf $
			, TITLE=title
	mplotw, /CLOSE
; ENDELSE
result = file + ' PLOTTED !'

IF !version.os_family EQ 'unix' THEN $
	SPAWN, 'ps2pdf '+ file +'.check.ps '+ file+'.check.pdf '

err=0
RETURN, result
END


FUNCTION edf_checker_2, file, REMOVE_EDF=remove_edf
_edf_utils_def_color_sysv2

path = FILE_DIRNAME(file, /MARK_DIRECTORY)
name = FILE_BASENAME(file)
; some startup parameters
in_path  = path & out_path = path & log_path = path

; logfile for messages to operator, not for user
logfile = log_path + 'edf_checker.log'
info = edf_checker_date(VERSION=v, TIME=t, COPYRIGHT=c)
logtext = ['!! edf_checker --- version: '+ v + ' created '+t $
	, '!! edf_checker --- copyright: '+ c , '!! ']
icg_message, logtext, FILE=logfile, /_TIME
logtext = [logtext, '!! edf_checker --- filename: '+file]
icg_message, logtext[1], FILE=logfile, /_TIME

; test access to the file
IF NOT FILE_TEST(file, /REG, /READ) THEN BEGIN
	icg_message, 'no access to: '+file, FILE=logfile, /_TIME
	RETURN, 92
ENDIF

; check the file for errors and write result to file
t0 = SYSTIME(1)
logtext = [logtext, '!! edf_checker --- started check at: '+js2string(/NOW), '!! ']
result = STRTRIM(check_edf(file, ERROR = err),2)
logtext2 = ['!! ','!! edf_checker --- finished check at: '+js2string(/NOW) $
	, '!! edf_checker --- time used for check: '+STRTRIM(STRING(SYSTIME(1)-t0, FORM='(F8.3)'),2)+' s' , '!! ']
_edf_utils_put_file, file+'.check.log', [logtext, result, logtext2]
icg_message, [logtext, result], FILE=logfile
IF err NE 0 THEN BEGIN
	_edf_checker_html_error, file, [logtext, result, logtext2]
	IF KEYWORD_SET(remove_edf) THEN FILE_DELETE, file, /QUIET
	RETURN, 0
ENDIF

; read file into sructure
struct = read_edf(file, ERROR = err)
IF err NE 0 THEN BEGIN
	icg_message, 'read_edf failed: '+file, FILE=logfile, /_TIME
	IF KEYWORD_SET(remove_edf) THEN FILE_DELETE, file, /QUIET, /ALLOW
	RETURN, 93
ENDIF

; plot and analyze the file
r2 = _edf_checker_plot_struct(file, struct, ERROR = err)
icg_message, r2, FILE=logfile, /_TIME

r3 = _edf_checker_html_success(file, ERROR=err)
icg_message, r3, FILE=logfile, /_TIME

IF KEYWORD_SET(remove_edf) THEN FILE_DELETE, file, /QUIET, /ALLOW
RETURN, 0
END


PRO edf_checker
;
; idl -rt=edf_checker.sav -args dir/file.edf
;
; file incl path is given thru the command line arguments
; return code 91 on error
; edf_checker_2 returns error codes 92, 93
;
edf_utils
IF LMGR(/RUNTIME) THEN BEGIN
	file = COMMAND_LINE_ARGS(COUNT= c)
	IF c NE 1 THEN BEGIN
		info = edf_checker_date(VERSION=v, TIME=t, COPYRIGHT=c)
		PRINT, 'edf_checker '+v+' created '+t
		EXIT, STATUS = 91
	ENDIF
	stat = edf_checker_2(file, /REMOVE_EDF)
	EXIT, STATUS = stat
ENDIF ELSE BEGIN
	file = (FILE_SEARCH('./data/', '*hola.edf', COUNT =c))
	IF c EQ 0 THEN STOP
	FOR i =0, c-1 DO PRINT, edf_checker_2(file[i])
ENDELSE
END
